package com.me.pacman;

import java.util.Arrays;

public class RoundModifiers {

    private static final RoundModifiers[] ROUND_MODIFIERS;

    private float frightTime;

    private float pacmanSpeed;
    private float pacmanFrightSpeed;

    private float ghostSpeed;
    private float ghostFrightSpeed;
    private float ghostTunnelSpeed;

    private int elroy1DotsLeft;
    private float elroy1Speed;
    private int elroy2DotsLeft;
    private float elroy2Speed;

    private int forceLeaveSeconds;

    private float[] scatterChaseSwitchTimings;

    private static int cap(int round) {
        return Math.min(round, 20);
    }

    public static float getFrightTime(int round) {
        return ROUND_MODIFIERS[cap(round)].frightTime;
    }

    public static float getPacmanSpeed(int round) {
        return Constants.FULL_SPEED * ROUND_MODIFIERS[cap(round)].pacmanSpeed;
    }

    public static float getPacmanFrightSpeed(int round) {
        return Constants.FULL_SPEED * ROUND_MODIFIERS[cap(round)].pacmanFrightSpeed;
    }

    public static float getGhostSpeed(int round) {
        return Constants.FULL_SPEED * ROUND_MODIFIERS[cap(round)].ghostSpeed;
    }

    public static float getGhostFrightSpeed(int round) {
        return Constants.FULL_SPEED * ROUND_MODIFIERS[cap(round)].ghostFrightSpeed;
    }

    public static float getGhostTunnelSpeed(int round) {
        return Constants.FULL_SPEED * ROUND_MODIFIERS[cap(round)].ghostTunnelSpeed;
    }

    public static int getElroy1DotsLeft(int round) {
        return ROUND_MODIFIERS[cap(round)].elroy1DotsLeft;
    }

    public static float getElroy1Speed(int round) {
        return Constants.FULL_SPEED * ROUND_MODIFIERS[cap(round)].elroy1Speed;
    }

    public static int getElroy2DotsLeft(int round) {
        return ROUND_MODIFIERS[cap(round)].elroy2DotsLeft;
    }

    public static float getElroy2Speed(int round) {
        return Constants.FULL_SPEED * ROUND_MODIFIERS[cap(round)].elroy2Speed;
    }

    public static int getForceLeaveSeconds(int round) {
        return ROUND_MODIFIERS[cap(round)].forceLeaveSeconds;
    }

    public static float getScatterChaseTimer(int round, int scatterChaseTransition) {
        return ROUND_MODIFIERS[cap(round)].scatterChaseSwitchTimings[scatterChaseTransition];
    }

    public RoundModifiers clone() {
        RoundModifiers mod = new RoundModifiers();
        mod.frightTime = frightTime;
        mod.pacmanSpeed = pacmanSpeed;
        mod.pacmanFrightSpeed = pacmanFrightSpeed;

        mod.ghostSpeed = ghostSpeed;
        mod.ghostFrightSpeed = ghostFrightSpeed;
        mod.ghostTunnelSpeed = ghostTunnelSpeed;

        mod.elroy1DotsLeft = elroy1DotsLeft;
        mod.elroy1Speed = elroy1Speed;
        mod.elroy2DotsLeft = elroy2DotsLeft;
        mod.elroy2Speed = elroy2Speed;

        mod.forceLeaveSeconds = forceLeaveSeconds;

        mod.scatterChaseSwitchTimings = Arrays.copyOf(scatterChaseSwitchTimings, scatterChaseSwitchTimings.length);
        return mod;
    }

    static {
        ROUND_MODIFIERS = new RoundModifiers[21];
        int round = 0;

        // Round 1
        RoundModifiers mod = new RoundModifiers();
        mod.frightTime = 6f;
        mod.pacmanSpeed = 0.8f;
        mod.pacmanFrightSpeed = 0.9f;

        mod.ghostSpeed = 0.75f;
        mod.ghostFrightSpeed = 0.5f;
        mod.ghostTunnelSpeed = 0.4f;

        mod.elroy1DotsLeft = 20;
        mod.elroy1Speed = 0.8f;
        mod.elroy2DotsLeft = 10;
        mod.elroy2Speed = 0.85f;

        mod.forceLeaveSeconds = 4;
        mod.scatterChaseSwitchTimings = new float[] {7f, 20f, 7f, 20f, 5f, 20f, 5f};

        ROUND_MODIFIERS[round++] = mod;

        // Round 2
        mod = mod.clone();
        mod.frightTime = 5f;
        mod.pacmanSpeed = 0.9f;
        mod.pacmanFrightSpeed = 0.95f;

        mod.ghostSpeed = 0.85f;
        mod.ghostFrightSpeed = 0.55f;
        mod.ghostTunnelSpeed = 0.45f;

        mod.elroy1DotsLeft = 30;
        mod.elroy1Speed = 0.9f;
        mod.elroy2DotsLeft = 14;
        mod.elroy2Speed = 0.95f;

        mod.scatterChaseSwitchTimings = new float[] {7f, 20f, 7f, 20f, 5f, 1033f, 1/60f};

        ROUND_MODIFIERS[round++] = mod;

        // Round 3
        mod = mod.clone();
        mod.frightTime = 4f;

        mod.elroy1DotsLeft = 40;
        mod.elroy1Speed = 0.9f;
        mod.elroy2DotsLeft = 20;
        mod.elroy2Speed = 0.95f;

        ROUND_MODIFIERS[round++] = mod;

        // Round 4
        mod = mod.clone();
        mod.frightTime = 3f;

        ROUND_MODIFIERS[round++] = mod;

        // Round 5
        mod = mod.clone();
        mod.frightTime = 2f;

        mod.pacmanSpeed = 1f;
        mod.pacmanFrightSpeed = 1f;

        mod.ghostSpeed = 0.95f;
        mod.ghostFrightSpeed = 0.6f;
        mod.ghostTunnelSpeed = 0.5f;

        mod.elroy1Speed = 1f;
        mod.elroy2Speed = 1.05f;

        mod.forceLeaveSeconds = 3;
        mod.scatterChaseSwitchTimings = new float[] {7f, 20f, 7f, 20f, 5f, 1037f, 1/60f};

        ROUND_MODIFIERS[round++] = mod;

        // Round 6
        mod = mod.clone();
        mod.frightTime = 5f;

        mod.elroy1DotsLeft = 50;
        mod.elroy2DotsLeft = 25;

        ROUND_MODIFIERS[round++] = mod;

        // Round 7 & 8
        mod = mod.clone();
        mod.frightTime = 2f;

        ROUND_MODIFIERS[round++] = mod;
        ROUND_MODIFIERS[round++] = mod;

        // Round 9
        mod = mod.clone();
        mod.frightTime = 1f;

        mod.elroy1DotsLeft = 60;
        mod.elroy2DotsLeft = 30;

        ROUND_MODIFIERS[round++] = mod;

        // Round 10
        mod = mod.clone();
        mod.frightTime = 5f;

        ROUND_MODIFIERS[round++] = mod;

        // Round 11
        mod = mod.clone();
        mod.frightTime = 2f;

        ROUND_MODIFIERS[round++] = mod;

        // Round 12 & 13
        mod = mod.clone();
        mod.frightTime = 1f;

        mod.elroy1DotsLeft = 80;
        mod.elroy2DotsLeft = 40;

        ROUND_MODIFIERS[round++] = mod;
        ROUND_MODIFIERS[round++] = mod;

        // Round 14
        mod = mod.clone();
        mod.frightTime = 3f;

        ROUND_MODIFIERS[round++] = mod;

        // Round 15 & 16
        mod = mod.clone();
        mod.frightTime = 1f;

        mod.elroy1DotsLeft = 100;
        mod.elroy2DotsLeft = 50;

        ROUND_MODIFIERS[round++] = mod;
        ROUND_MODIFIERS[round++] = mod;

        // Round 17
        mod = mod.clone();
        mod.frightTime = 0f;

        ROUND_MODIFIERS[round++] = mod;

        // Round 18
        mod = mod.clone();
        mod.frightTime = 1f;

        ROUND_MODIFIERS[round++] = mod;

        // Round 19 & 20
        mod = mod.clone();
        mod.frightTime = 0f;

        mod.elroy1DotsLeft = 120;
        mod.elroy2DotsLeft = 60;

        ROUND_MODIFIERS[round++] = mod;
        ROUND_MODIFIERS[round++] = mod;

        // Round 21+
        mod = mod.clone();
        mod.pacmanSpeed = 0.9f;

        ROUND_MODIFIERS[round++] = mod;
    }

}
