package com.me.pacman;

import java.util.Arrays;

public class SoundManager {

    public enum Effect {
        BEGINNING,
        BEGINNING_ALT,
        FRIGHT,
        CHOMP_1,
        CHOMP_2,
        DEATH,
        EAT_FRUIT,
        EAT_GHOST,
        EXTRA_LIFE,
        RETURN_BASE,
        SIREN,
        SIREN_FAST,
        SIREN_FASTER,
        SIREN_FASTEST
    }

    private float volume = 1.0f;
    private boolean muted;

    private PacDude game;
    private long[] playing;
    private long[] looping;

    public SoundManager(PacDude game) {
        this.game = game;
        this.playing = new long[Effect.values().length];
        this.looping = new long[Effect.values().length];
        Arrays.fill(this.playing, -1);
        Arrays.fill(this.looping, -1);
    }

    private float volume() {
        return muted? 0f : volume;
    }

    public void setMuted(boolean muted) {
        this.muted = muted;
    }

    public long play(Effect effect) {
        long id = getSound(effect).play(volume());
        playing[effect.ordinal()] = id;
        return id;
    }

    public long loop(Effect effect) {
        long id = getSound(effect).loop(volume());
        looping[effect.ordinal()] = id;
        return id;
    }

    public boolean isPlaying(Effect effect) {
        return playing[effect.ordinal()] > -1;
    }

    public boolean isLooping(Effect effect) {
        return looping[effect.ordinal()] > -1;
    }

    public void stopPlaying(Effect effect) {
        com.badlogic.gdx.audio.Sound sound = getSound(effect);
        long id = playing[effect.ordinal()];
        if (id > -1) {
            sound.stop(id);
            playing[effect.ordinal()] = -1;
        }
    }

    public void stopLooping(Effect effect) {
        com.badlogic.gdx.audio.Sound sound = getSound(effect);
        long id = looping[effect.ordinal()];
        if (id > -1) {
            sound.stop(id);
            looping[effect.ordinal()] = -1;
        }
    }

    public void stopPlays() {
        for (int i = 0; i < playing.length; i++) {
            if (playing[i] > -1) {
                getSound(Effect.values()[i]).stop(playing[i]);
                playing[i] = -1;
            }
        }
    }

    public void stopLoops() {
        for (int i = 0; i < looping.length; i++) {
            if (looping[i] > -1) {
                getSound(Effect.values()[i]).stop(looping[i]);
                looping[i] = -1;
            }
        }
    }

    public void stopAll() {
        stopPlays();
        stopLoops();
    }

    public void pauseAll() {
        for (int i = 0, n = Effect.values().length; i < n; i++) {
            com.badlogic.gdx.audio.Sound sound = getSound(Effect.values()[i]);
            if (playing[i] > -1) {
                sound.pause(playing[i]);
            }
            if (looping[i] > -1) {
                sound.pause(looping[i]);
            }
        }
    }

    public void resumeAll() {
        for (int i = 0, n = Effect.values().length; i < n; i++) {
            com.badlogic.gdx.audio.Sound sound = getSound(Effect.values()[i]);
            if (playing[i] > -1) {
                sound.resume(playing[i]);
            }
            if (looping[i] > -1) {
                sound.resume(looping[i]);
            }
        }
    }

    private com.badlogic.gdx.audio.Sound getSound(Effect effect) {
        Assets a = game.assets;
        switch (effect) {
            case BEGINNING:
                return a.beginning;
            case BEGINNING_ALT:
                return a.beginning_alt;
            case FRIGHT:
                return a.fright;
            case CHOMP_1:
                return a.chomp_1;
            case CHOMP_2:
                return a.chomp_2;
            case DEATH:
                return a.deathSound;
            case EAT_FRUIT:
                return a.eat_fruit;
            case EAT_GHOST:
                return a.eat_ghost;
            case EXTRA_LIFE:
                return a.extra_life;
            case RETURN_BASE:
                return a.return_base;
            case SIREN:
                return a.siren;
            case SIREN_FAST:
                return a.siren_fast;
            case SIREN_FASTER:
                return a.siren_faster;
            case SIREN_FASTEST:
                return a.siren_fastest;
        }
        return null;
    }

}
