package com.me.pacman.level;

import com.badlogic.gdx.graphics.g2d.TextureRegion;
import com.badlogic.gdx.math.Vector2;
import com.me.pacman.Constants;
import com.me.pacman.PacDude;

public class Level {

    public static final Vector2[] GHOST_SPAWN_POINTS = {
        new Vector2(14f, 19.5f),
        new Vector2(14f, 16.5f),
        new Vector2(12f, 16.5f),
        new Vector2(16f, 16.5f),
    };

    private PacDude game;

    private TextureRegion pellet;
    private TextureRegion powerPellet;
    private TextureRegion debug;

    // Grid of tiles, [rows][columns]
    public LevelTile[][] tiles;

    // Level width and height in tiles
    public int width;
    public int height;

    private int pellets;
    private int pelletsRemaining;

    public Level(PacDude game, String level) {
        this.game = game;

        pellet = game.assets.level[0][6];
        powerPellet = game.assets.level[0][7];
        debug = game.assets.level[1][7];

        LevelLoader loader = new LevelLoader(level);
        tiles = loader.loadLevel();

        height = tiles.length;
        width = tiles[0].length;

        pellets = pelletsRemaining = getTileCount(LevelTile.PELLET) + getTileCount(LevelTile.POWER_PELLET);
    }

    public int getPelletCount() {
        return pellets;
    }

    public int getPelletsRemaining() {
        return pelletsRemaining;
    }

    public int getPelletsEaten() {
        return pellets - pelletsRemaining;
    }

    public LevelTile getTile(int x, int y) {
        return y >= 0 && y < height && x >= 0 && x < width ? tiles[y][x] : null;
    }

    public LevelTile getTile(float x, float y) {
        return getTile((int) x, (int) y);
    }

    public LevelTile getTile(Vector2 vec) {
        return getTile(vec.x, vec.y);
    }

    public void setTile(int x, int y, LevelTile tile) {
        LevelTile exist = tiles[y][x];
        if (exist.isPellet() && !tile.isPellet()) {
            pelletsRemaining--;
        }
        tiles[y][x] = tile;
    }

    public void setTile(float x, float y, LevelTile tile) {
        setTile((int) x, (int) y, tile);
    }

    public int getTileCount(LevelTile tile) {
        int count = 0;
        for (int i = 0; i < tiles.length; i++) {
            LevelTile[] row = tiles[i];
            for (int j = 0; j < row.length; j++) {
                if (row[j] == tile) {
                    count++;
                }
            }
        }
        return count;
    }

    public void render(int offsetX, int offsetY) {
        for (int i = 0; i < tiles.length; i++) {
            LevelTile[] row = tiles[i];
            for (int j = 0; j < row.length; j++) {
                renderTile(row[j], j, i, offsetX, offsetY);
            }
        }
    }

    public void renderTile(LevelTile tile, int tileX, int tileY, int offsetX, int offsetY) {
        TextureRegion sprite;
        switch (tile) {
            case PELLET:
                sprite = pellet;
                break;
            case POWER_PELLET:
                sprite = powerPellet;
                break;
            case DEBUG:
                sprite = debug;
                break;
            default:
                return;
        }
        game.batch.draw(sprite, (tileX * Constants.TILE_SIZE) + offsetX, (tileY * Constants.TILE_SIZE) + offsetY);
    }

}
